@@ -14,15 +14,12 @@ package LaTeXML::Post::LaTeXImages;
 use strict;
 use warnings;
 use DB_File;
-#use Image::Magick;
-#use Graphics::Magick;
+use Config;
 use LaTeXML::Util::Image;
-use POSIX;
+use POSIX qw(ceil);
 use LaTeXML::Util::Pathname;
-use File::Temp qw(tempdir);
-use File::Path;
+use File::Temp;
 use File::Which;
-use FindBin;
 use LaTeXML::Post;
 use base qw(LaTeXML::Post::Processor);
 
@@ -38,17 +35,6 @@ our $LATEXCMD = 'latex';    #(or elatex) [ CONFIGURABLE? Encoded in PI?]
 # dvips converts the dvi to eps, and ImageMagick can convert the eps to png;
 # OR dvipng can convert the dvi to png MUCH quicker... if it's available.
 
-# Useful DVIPS options:
-#  -q  : run quietly
-#  -x#  : magnification * 1000
-#  -S1 -i  : make a separate file for each `section' consisting of a single page.
-#       QUESTION: dvips' naming scheme allows for 999 pages... what happens at 1000?
-#  -E   :  crop each page close to the `ink'.
-#  -j0  : don't subset fonts; silly really, but some font tests are making problems!
-
-our $DVIPSCMD  = 'dvips -q -S1 -i -E -j0 -o imgx';                    # [ CONFIGURABLE?]
-our $DVIPNGCMD = 'dvipng -bg Transparent -T tight -q -o imgx%03d';    # [ CONFIGURABLE?]
-
 # Options:
 #   source         : (dir)
 #   magnification  : typically something like 1.33333, but you may want bigger
@@ -60,11 +46,10 @@ sub new {
   my ($class, %options) = @_;
   my $self = $class->SUPER::new(%options);
   $$self{magnification} = $options{magnification} || 1.33333;
-  $$self{magnification} = 1.3333333;
-  $$self{maxwidth}      = $options{maxwidth} || 800;
-  $$self{dpi}           = $options{dpi} || 96;
-  $$self{background}    = $options{background} || "#FFFFFF";
-  $$self{imagetype}     = $options{imagetype} || 'png';
+  $$self{maxwidth}      = $options{maxwidth}      || 800;
+  $$self{dpi}           = $options{dpi}           || 96;
+  $$self{background}    = $options{background}    || "#FFFFFF";
+  $$self{imagetype}     = $options{imagetype}     || 'png';
 
   # Parameters for separating the clipping box from the
   # desired padding between image edge and "ink"
@@ -73,12 +58,61 @@ sub new {
   $$self{clippingfudge} = 3;       # px
   $$self{clippingrule}  = 0.90;    # pixels (< 1 to avoid antialiasing..?)
 
-  # We'll use dvipng (MUCH faster) if requested or not forbidden & available.
-  # But be careful: it can't handle much postscript, so better NOT for graphics!
-  $$self{use_dvipng} = 1
-    if ($options{use_dvipng} || (!defined $options{use_dvipng})) && which('dvipng');
-  $$self{dvicmd}             = ($$self{use_dvipng} ? $DVIPNGCMD : $DVIPSCMD);
-  $$self{dvicmd_output_type} = ($$self{use_dvipng} ? 'png32'    : 'eps');
+  # Sanity check of dvi processing...
+  # If trying to create svg...
+  if (($$self{imagetype} eq 'svg') && (!defined $options{use_dvisvgm}) && which('dvisvgm')) {
+    $$self{use_dvisvgm} = 1; }
+  elsif ($$self{use_dvisvgm} && (($$self{imagetype} ne 'svg') || !which('dvisvgm'))) {
+    $$self{use_dvisvgm} = 0; }    # but disable if inappropriate or unavailable
+      # We'll use dvipng (MUCH faster) if requested or not forbidden & available.
+      # But be careful: it can't handle much postscript, so better NOT for graphics!
+  if (($$self{imagetype} eq 'png') && (!defined $options{use_dvipng}) && which('dvipng')) {
+    $$self{use_dvipng} = 1; }
+  elsif ($$self{use_dvipng} && (($$self{imagetype} ne 'png') || !which('dvipng'))) {
+    $$self{use_dvipng} = 0; }    # but disable if inappropriate or unavailable.
+
+  # Parameterize according to the selected dvi-to-whatever processor.
+  my $mag = int($$self{magnification} * 1000);
+  my $dpi = int($$self{dpi} * $$self{magnification});
+  # Unfortunately, each command has incompatible -o option to name the output file.
+  # Note that the formatting char used, '%', has to be doubled on Windows!!
+  # Maybe that's only true when it looks like an environment variable?
+  # Ie. %variable% ?
+  #  my $fmt = ($^O eq 'MSWin32' ? '%%' : '%');
+  my $fmt = '%';
+  if ($$self{use_dvisvgm}) {
+    # img name uses 2 digits for page; what happens at 100?
+    ##    $$self{dvicmd}             = "dvisvgm --page=1- --bbox=min --mag=$mag -o imgx-${fmt}p";
+    # dvisvgm currently creates glyph descriptions w/ unicode attribute having the wrong codepoint
+    # firefox, chromium use this codepoint instead of the glyph "drawing"
+    # a later version of dvisvgm should do better at synthesizing the unicode?
+    # but for now, we'll use --no-fonts, which creates glyph drawings rather than "glyphs"
+    # Also, increase the bounding box from min by 1pt
+    # Also, note incompatible change in the -o option after version 1.6;
+    # it now can take a number of digits (%3p) for pages, BUT old version produces imgx-%3p.svg!!
+    # So, simply use %p, let new version do 1,..,9,10,...
+    # Recovery code below!
+    $$self{dvicmd} = "dvisvgm --page=1- --bbox=1pt --scale=$$self{magnification} --no-fonts -o imgx-${fmt}p";
+    $$self{dvicmd_output_name} = 'imgx-%02d.svg';
+    $$self{dvicmd_output_type} = 'svg';
+    $$self{frame_output}       = 0; }
+  elsif ($$self{use_dvipng}) {
+    $$self{dvicmd}             = "dvipng -bg Transparent -T tight -q -D$dpi -o imgx-${fmt}03d.png";
+    $$self{dvicmd_output_name} = 'imgx-%03d.png';
+    $$self{dvicmd_output_type} = 'png32';
+    $$self{frame_output}       = 1; }
+  else {
+    # Useful DVIPS options:
+    #  -q  : run quietly
+    #  -x#  : magnification * 1000
+    #  -S1 -i  : make a separate file for each `section' consisting of a single page.
+    #       QUESTION: dvips' naming scheme allows for 999 pages... what happens at 1000?
+    #  -E   :  crop each page close to the `ink'.
+    #  -j0  : don't subset fonts; silly really, but some font tests are making problems!
+    $$self{dvicmd}             = "dvips -q -S1 -i -E -j0 -x$mag -o imgx";
+    $$self{dvicmd_output_name} = 'imgx%03d';
+    $$self{dvicmd_output_type} = 'eps';
+    $$self{frame_output}       = 1; }
   return $self; }
 
 #**********************************************************************
@@ -148,8 +182,9 @@ sub cleanTeX {
     $style = $1; }
   $tex =~ s/^(?:\\\s*,|\\!\s*|\\>\s*|\\;\s*|\\:\s*|\\ \s*|\\\/\s*)*//; # Trim leading spacing (especially negative!)
   $tex =~ s/(?:\\\s*,|\\!\s*|\\>\s*|\\;\s*|\\:\s*|\\ \s*|\\\/\s*)*$//; # and trailing spacing
-  $tex =~ s/\%[^\n]*\n//gs;                                            # Strip comments
-  $tex = $style . ' ' . $tex if $style;                                # Put back the style, if any
+      # Strip comments, but watchout for \% (or more exactly, an ODD number of \)
+  $tex =~ s/(?<!\\)((?:\\\\)*)\%[^\n]*\n/$1/gs;
+  $tex = $style . ' ' . $tex if $style;    # Put back the style, if any
   return $tex; }
 
 #**********************************************************************
@@ -164,10 +199,19 @@ sub cleanTeX {
 #   (1) where to write the images.
 #   (2) relative path from doc to images.
 
-sub process {
+# sub process {
+#   my ($self, $doc, @nodes) = @_;
+#   return $self->generateImages($doc, @nodes); }
+
+sub generateImages {
   my ($self, $doc, @nodes) = @_;
 
-  my $jobname = "ltxmlimg";
+  my $jobname  = "ltxmlimg";
+  my $orig_cwd = pathname_cwd();
+  my $sep      = $Config::Config{path_sep};
+  local $ENV{TEXINPUTS} = join($sep, '.', $doc->getSearchPaths,
+    pathname_concat(pathname_installation(), 'texmf'),
+    ($ENV{TEXINPUTS} || $sep));
 
   my %table = ();
 
@@ -183,11 +227,17 @@ sub process {
 
     # Ideally, $dest is relative to document..
     my $dest = $self->desiredResourcePathname($doc, $node, undef, $$self{imagetype});
-    my $key = (ref $self) . ":" . $tex . ($dest ? ":$dest" : '');
+    #    my $key = (ref $self) . ":" . $tex . ($dest ? ":$dest" : '');
+    my $key   = (ref $self) . ':' . $$self{imagetype} . ':' . $tex;
     my $entry = $table{$key};
     if (!$entry) {
       $nuniq++;
-      $entry = $table{$key} = { tex => $tex, key => $key, nodes => [], dest => $dest }; }
+      $entry = $table{$key} = { tex => $tex, key => $key, nodes => [], dest => [] }; }
+    # Why do I need to make things so complicated?!?!
+    # It may be desired that a particular image gets a specific name.
+    # AND, the same TeX may get different names, or some with & without names!!
+    if ($dest && (!grep { $dest eq $_ } @{ $$entry{dest} })) {
+      push(@{ $$entry{dest} }, $dest); }
     push(@{ $$entry{nodes} }, $node); }
 
   return $doc unless $nuniq;    # No strings to process!
@@ -207,14 +257,14 @@ sub process {
   if (@pending) {    # if any images need processing
         # Create working directory; note TMPDIR attempts to put it in standard place (like /tmp/)
     File::Temp->safe_level(File::Temp::HIGH);
-    my $workdir = tempdir("LaTeXMLXXXXXX", CLEANUP => 0, TMPDIR => 1);
-    my $preserve_tmpdir = 0;
+    my $workdir = File::Temp->newdir("LaTeXMLXXXXXX", TMPDIR => 1);
     # === Generate the LaTeX file.
     my $texfile = pathname_make(dir => $workdir, name => $jobname, type => 'tex');
     my $TEX;
     if (!open($TEX, '>', $texfile)) {
       Error('I/O', $texfile, undef, "Cant write to '$texfile'", "Response was: $!");
       return $doc; }
+    binmode $TEX, ':encoding(UTF-8)';
     print $TEX $self->pre_preamble($doc);
     print $TEX "\\makeatletter\n";
     print $TEX $self->preamble($doc) . "\n";
@@ -227,24 +277,23 @@ sub process {
     close($TEX);
 
     # === Run LaTeX on the file.
-    my $texinputs = ".:" . join(':', $doc->getSearchPaths, "$FindBin::RealBin/../lib/LaTeXML/texmf/")
-      . ":" . ($ENV{TEXINPUTS} || '');
-    my $command = "cd $workdir ; TEXINPUTS=$texinputs $LATEXCMD $jobname > $jobname.output";
-    my $err     = system($command);
+    # (keep the command simple so it works in Windows)
+    pathname_chdir($workdir);
+    my $ltxcommand = "$LATEXCMD $jobname > $jobname.ltxoutput";
+    my $ltxerr     = system($ltxcommand);
+    pathname_chdir($orig_cwd);
 
     # Sometimes latex returns non-zero code, even though it apparently succeeded.
-    if ($err != 0) {
-      $preserve_tmpdir = 1;
-      Error('shell', $command, undef,
-        "Shell command ($command) returned code $err (!= 0) for image generation",
-        "Response was: $@", "See $workdir/$jobname.log"); }
-    if (!-f "$workdir/$jobname.dvi") {
-      $preserve_tmpdir = 1;
-      Error('shell', $command, undef,
-        "Shell command '$command' (for latex) failed: See $workdir/$jobname.log");
+    # And sometimes it doesn't produce a dvi, even with 0 return code?
+    if (($ltxerr != 0) || (!-f "$workdir/$jobname.dvi")) {
+      my $preserve = $$LaTeXML::POST{verbosity} > 0;    # preserve junk when verbosity high.
+      $workdir->unlink_on_destroy(0) if $preserve;
+      Error('shell', $LATEXCMD, undef,
+        "LaTeX command '$ltxcommand' failed",
+        ($ltxerr == 0 ? "No dvi file generated"     : "returned code $ltxerr (!= 0): $@"),
+        ($preserve    ? "See $workdir/$jobname.log" : "Re-run with --verbose to see TeX log"));
       return $doc; }
-
-    $preserve_tmpdir = 1 if $$LaTeXML::POST{verbosity} > 2;
+    $workdir->unlink_on_destroy(0) if $$LaTeXML::POST{verbosity} > 2;
 
     # Extract dimensions (width x height+depth) from each image from log file.
     my @dimensions = ();
@@ -260,36 +309,47 @@ sub process {
         "Response was: $!"); }
 
     # === Run dvicmd to extract individual png|postscript files.
-    my $mag           = int($$self{magnification} * 1000);
-    my $pixels_per_pt = $$self{magnification} * $$self{dpi} / 72.27;
-    my $dpi           = int($$self{dpi} * $$self{magnification});
-    my $resoption     = ($$self{use_dvipng} ? "-D$dpi" : "-x$mag");
-    if (system("cd $workdir ; TEXINPUTS=$texinputs $$self{dvicmd} $resoption $jobname.dvi") != 0) {
+    pathname_chdir($workdir);
+    my $dvicommand = "$$self{dvicmd} $jobname.dvi > $jobname.dvioutput";
+    my $dvierr     = system($dvicommand);
+    pathname_chdir($orig_cwd);
+
+    if ($dvierr != 0) {
       Error('shell', $$self{dvicmd}, undef,
-        "Shell command $$self{dvicmd} (for dvi conversion) failed (see $workdir for clues)",
+        "Shell command '$dvicommand' (for dvi conversion) failed (see $workdir for clues)",
         "Response was: $!");
       return $doc; }
-
     # === Convert each image to appropriate type and put in place.
+    my $pixels_per_pt = $$self{magnification} * $$self{dpi} / 72.27;
     my ($index, $ndigits) = (0, 1 + int(log($doc->cacheLookup((ref $self) . ':_max_image_') || 1) / log(10)));
     foreach my $entry (@pending) {
-      my $src = "$workdir/imgx" . sprintf("%03d", ++$index);
+      my $src = "$workdir/" . sprintf($$self{dvicmd_output_name}, ++$index);
+      # Recovery patchup for incompatible change to dvisvgm!
+      if (($index == 1) && (!-f $src) && (-f "$workdir/imgx-1.svg")) {
+        $$self{dvicmd_output_name} = 'imgx-%d.svg';
+        $src = "$workdir/imgx-1.svg"; }
       if (-f $src) {
-        my $dest = $$entry{dest}
-          || $self->generateResourcePathname($doc, $$entry{nodes}[0], undef, $$self{imagetype});
-        my $absdest = $doc->checkDestination($dest);
-        my ($w, $h) = $self->convert_image($doc, $src, $absdest);
-        next unless defined $w && defined $h;
-        my ($ww, $hh, $dd) = map { $_ * $pixels_per_pt } @{ $dimensions[$index] };
-        my $d = int(0.5 + ($dd || 0) + $$self{padding});
-        if ((($w == 1) && ($ww > 1)) || (($h == 1) && ($hh > 1))) {
-          Warn('expected', 'image', undef, "Image for '$$entry{tex}' was cropped to nothing!"); }
-        # print STDERR "\nImage[$index] $$entry{tex} $ww x $hh + $dd ==> $w x $h \\ $d\n";
-        $doc->cacheStore($$entry{key}, "$dest;$w;$h;$d"); }
+        my @dests = @{ $$entry{dest} };
+        push(@dests, $self->generateResourcePathname($doc, $$entry{nodes}[0], undef, $$self{imagetype}))
+          unless @dests;
+        foreach my $dest (@dests) {
+          my $absdest = $doc->checkDestination($dest);
+          my ($ww, $hh, $dd) = map { $_ * $pixels_per_pt } @{ $dimensions[$index] };
+          my ($w, $h);
+          if ($$self{frame_output}) {    # If framed, trim the frame
+            ($w, $h) = $self->convert_image($doc, $src, $absdest);
+            next unless defined $w && defined $h; }
+          else {
+            pathname_copy($src, $absdest);
+            $w = int($ww + 0.5); $h = int($hh + $dd + 0.5); }
+          my $d = int(0.5 + ($dd || 0) + $$self{padding});
+          if ((($w == 1) && ($ww > 1)) || (($h == 1) && ($hh > 1))) {
+            Warn('expected', 'image', undef, "Image for '$$entry{tex}' was cropped to nothing!"); }
+#          print STDERR "\nFor key='$$entry{key}': Image[$index] '$dest' $$entry{tex} $ww x $hh + $dd ==> $w x $h \\ $d\n";
+          $doc->cacheStore($$entry{key}, "$dest;$w;$h;$d"); }
+      }
       else {
-        Warn('expected', 'image', undef, "Missing image '$src'; See $workdir/$jobname.log"); } }
-    # Cleanup
-    rmtree($workdir) unless $preserve_tmpdir; }
+        Warn('expected', 'image', undef, "Missing image '$src'; See $workdir/$jobname.log"); } } }
 
   # Finally, modify the original document to record the associated images.
   foreach my $entry (values %table) {
@@ -299,31 +359,10 @@ sub process {
     my $reldest = pathname_relative($image, $doc->getDestinationDirectory);
     foreach my $node (@{ $$entry{nodes} }) {
       $self->setTeXImage($doc, $node, $reldest, $width, $height, $depth); } }
+
   $doc->closeCache;    # If opened.
   return $doc; }
 
-# Get a list blah, blah...
-sub find_documentclass_and_packages {
-  my ($self, $doc) = @_;
-  my ($class, $classoptions, $oldstyle, @packages);
-  foreach my $pi ($doc->findnodes(".//processing-instruction('latexml')")) {
-    my $data  = $pi->textContent;
-    my $entry = {};
-    while ($data =~ s/\s*([\w\-\_]*)=([\"\'])(.*?)\2//) {
-      $$entry{$1} = $3; }
-    if ($$entry{class}) {
-      $class        = $$entry{class};
-      $classoptions = $$entry{options} || 'onecolumn';
-      $oldstyle     = $$entry{oldstyle}; }
-    elsif ($$entry{package}) {
-      push(@packages, [$$entry{package}, $$entry{options} || '']); }
-  }
-  if (!$class) {
-    Warn('expected', 'class', undef, "No document class found; using article");
-    $class = 'article'; }
-
-  return ([$class, $classoptions, $oldstyle], @packages); }
-
 #======================================================================
 # Generating & Processing the LaTeX source.
 #======================================================================
@@ -346,8 +385,10 @@ sub pre_preamble {
 
   my $w   = ceil($$self{maxwidth} * $pts_per_pixel);                      # Page Width in points.
   my $gap = ($$self{padding} + $$self{clippingfudge}) * $pts_per_pixel;
-  my $th = $$self{clippingrule} * $pts_per_pixel;   # clipping box thickness in points.
-                                                    #  print STDERR "w=$w, gap=$gap, thickness=$th\n";
+  my $th  = ($$self{frame_output}
+    ? $$self{clippingrule} * $pts_per_pixel    # clipping box thickness in points.
+    : 0);                                      # NO clipping box!
+                                               #  print STDERR "w=$w, gap=$gap, thickness=$th\n";
   return <<"EOPreamble";
 \\batchmode
 \\def\\inlatexml{true}
@@ -456,5 +497,14 @@ sub convert_image {
   $image->Write(filename => $dest);
   return ($w, $h); }
 
+sub DESTROY {
+  if (my $tmpdir = File::Spec->tmpdir()) {
+    if (-d $tmpdir && opendir(my $tmpdir_fh, $tmpdir)) {
+      my @empty_magick = grep { -z $_ } map { "$tmpdir/$_" } readdir($tmpdir_fh);
+      closedir($tmpdir_fh);
+      unlink $_ foreach @empty_magick;
+    } }
+  return; }
+
 #======================================================================
 1;
